@@ -1,5 +1,13 @@
 package Carp;
 
+our (@ISA, @EXPORT, @EXPORT_OK, @EXPORT_FAIL);
+BEGIN {
+    @EXPORT    = qw(confess croak carp);
+    @EXPORT_OK = qw(cluck verbose longmess shortmess);
+    @EXPORT_FAIL = qw(verbose);    # hook to enable verbose mode
+}
+use Exporter qw(import);
+
 #       carp    - warn of errors (from perspective of caller)
 sub carp {
     warn @_;
@@ -21,26 +29,5 @@ sub confess {
     die @_;
 }
 
-sub import {
-    my $self = shift;
-    my ($pkg) = caller();
-    my @exports = @_;
-
-    push @exports, 'carp'
-        unless grep { $_ eq 'carp' } @exports;
-    push @exports, 'croak'
-        unless grep { $_ eq 'croak' } @exports;
-
-    # print "called from $pkg [ @exports ]\n";
-
-    for my $export (@exports) {
-        *{ $pkg . '::' . $export } = \&{ $export };
-    }
-
-}
-
-sub unimport {
-}
-
 1;
 
